/*	$AoiMoe: file.h,v 1.5 2001/08/08 19:25:20 tshiozak Exp $	*/

/*
 * Copyright (c) 2001 Takuya SHIOZAKI <AoiMoe@imou.to>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


#ifndef AMUSE_FILE_
#define AMUSE_FILE_

class IFIleMap;
class IFIlePtr;

// unix version

class IFile {
	friend class IFilePtr;
protected:
	void *head_;
	size_t size_;
public:
	IFile(char const *);
	~IFile();
	void *map(size_t pos, size_t sz) const
	{
		if (pos+sz > size_)
			return NULL;
		return (uint8_t *)head_ + pos;
	}
	void unmap(void *) const  { }
	operator bool () const { return !!head_; }
	size_t size() const { return size_; }
};

class IFileMap {
	friend class IFilePtr;
	IFile const *file_;
	void *page_;
	IFileMap(IFile const *f, void *p) : file_(f), page_(p) { }
public:
	IFileMap() : file_(0), page_(0) { }
	operator void * () const { return page_; }
	uint8_t operator [] (int p) const { return *((uint8_t *)page_ + p); }
	~IFileMap() { if (page_)  file_->unmap(page_); }
};

class IFilePtr {
	IFile const *file_;
	size_t curpos_;
public:
	IFilePtr(IFile const *f=NULL) : file_(f), curpos_(0) { }
	void bind(IFile const *f) { file_ = f; }
	IFileMap map(size_t sz)
	{
		IFileMap m;

		if (!file_)
			return m;
		m.file_ = file_;
		m.page_ = file_->map(curpos_, sz);
		if (m.page_)
			curpos_ += sz;
		return m;
	}
	size_t remain() const { return file_->size_ - curpos_; }
	void rewind() { curpos_ = 0; }
	void fwd(size_t sz) { curpos_ += remain()>=sz ? sz:remain(); }
	uint8_t peek8()
	{
		IFileMap m(file_, file_->map(curpos_, 1));
		return m[0];
	}
	uint8_t get8()
	{
		uint8_t v;
		v = peek8();
		curpos_++;
		return v;
	}
	uint16_t get16le()
	{
		IFileMap m(file_, file_->map(curpos_, 2));
		curpos_+=2;
		return m[0] | uint16_t(m[1])<<8;
	}
	uint16_t get16be()
	{
		IFileMap m(file_, file_->map(curpos_, 2));
		curpos_+=2;
		return m[1] | (uint16_t(m[0])<<8);
	}
	uint32_t get32le()
	{
		IFileMap m(file_, file_->map(curpos_, 4));
		curpos_+=4;
		return
		    m[0] | (uint32_t(m[1])<<8) |
		    (uint32_t(m[2])<<16) | (uint32_t(m[3])<<24);
	}
	uint32_t get32be()
	{
		IFileMap m(file_, file_->map(curpos_, 4));
		curpos_+=4;
		return
		    m[3] | (uint32_t(m[2])<<8) |
		    (uint32_t(m[1])<<16) | (uint32_t(m[0])<<24);
	}
};

#endif
