/*	$AoiMoe: list.h,v 1.9 2001/08/01 20:03:23 tshiozak Exp $	*/

/*
 * Copyright (c) 2001 Takuya SHIOZAKI <AoiMoe@imou.to>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


#ifndef AMUSE_LIST_H_
#define AMUSE_LIST_H_

template <typename T_>
class AMList {
	class Iterator;
	class NodeRec {
		friend AMList;
		friend Iterator;
		AMList *list_;
		NodeRec *next_, *prev_;
		T_ *own_;
	protected:
		NodeRec(T_ *o) : own_(o) { }
	public:
		T_ const &to_own() const { return own_; }
		NodeRec const *next() const { return own_->next_; }
		NodeRec const *prev() const { return own_->prev_; }
	};
	class Iterator {
		friend AMList;
		NodeRec const *p_;
		Iterator(NodeRec const *p) : p_(p) { }
	public:
		Iterator(AMList const &l) : p_(l.head_) { }
		Iterator() : p_(NULL) { }
		void operator ++ () { if (p_) p_ = p_->next_; }
		void operator -- () { if (p_) p_ = p_->prev_; }
		operator T_ * () const { return p_?p_->own_:0; }
		operator bool () const { return !!p_; }
		T_ *remove()
		{
			if (p_) {
				T_ *ret = p_->list_->remove(*this);
				p_ = NULL;
				return ret;
			} else
				return NULL;
		}
	};
	friend Iterator;
	NodeRec *head_, *tail_;
public:
	AMList() : head_(0), tail_(0) { }
	~AMList() { }
	void insert_head(T_ *o)
	{
		NodeRec *n = new NodeRec(o);
		if (head_)
			head_->prev_ = n;
		else
			tail_ = n;

		n->next_ = head_;
		n->prev_ = 0;
		n->list_ = this;
		head_ = n;
	}
	void insert_tail(T_ *o)
	{
		NodeRec *n = new NodeRec(o);
		if (tail_)
			tail_->next_ = n;
		else
			head_ = n;
		n->prev_ = tail_;
		n->next_ = 0;
		n->list_ = this;
		tail_ = n;
	}
	T_ *remove_head()
	{
		T_ *o=0;
		NodeRec *n;
		if (head_) {
			n = head_;
			if (!(head_ = n->next_))
				tail_ = 0;
			o = n->own_;
			delete n;
		}
		return o;
	}
	T_ *remove_tail()
	{
		T_ *o=0;
		NodeRec *n;
		if (tail_) {
			n = tail_;
			if (!(tail_ = n->prev_))
				head_ = 0;
			o = n->own_;
			delete n;
		}
		return o;
	}
private:
	T_ *remove(Iterator ni)
	{
		T_ *o=0;
		NodeRec *n = const_cast<NodeRec *>(ni.p_);
		if (n) {
			if (n->next_)
				n->next_->prev_ = n->prev_;
			else
				tail_ = n->prev_;
			if (n->prev_)
				n->prev_->next_ = n->next_;
			else
				head_ = n->next_;
			o = n->own_;
			delete n;
		}
		return 0;
	}
public:
	Iterator first() const { return Iterator(head_); }
	Iterator last() const { return Iterator(tail_); }
	bool isempty() const { return !head_; }
};


template <typename T_>
class AMQueue {
	AMList<T_> list_;
public:
	T_ const &peek() const { return *list_.first(); }
	T_ const &peek_tail() const { return *list_.tail(); }
	T_ *pop() { return list_.remove_head(); }
	void push(T_ *o) { return list_.insert_tail(o); }
	bool isempty() const { return list_.isempty(); }
};


#endif
