/*	$AoiMoe: lookup.cc,v 1.4 2001/08/01 20:03:23 tshiozak Exp $	*/

/*
 * Copyright (c) 2001 Takuya SHIOZAKI <AoiMoe@imou.to>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <errno.h>
#include <dirent.h>
#include "types.h"
#include "debug.h"
#include "lookup.h"


// ----------------------------------------------------------------------

class Dir {
	DIR *dir_;
	struct dirent *dirent_;
public:
	Dir() : dir_(NULL), dirent_(NULL) { }
	~Dir() { if (dir_) closedir(dir_); }
	void open(char const *dname)
	{
		if (dir_)
			closedir(dir_);
		dir_ = opendir(dname);
		if (dir_) {
			dirent_ = readdir(dir_);
		}
	}
	explicit Dir(char const *dir) :
		dir_(NULL), dirent_(NULL)
	{
		open(dir);
	}
	
	operator bool () const { return !!dirent_; }
	struct dirent * operator -> () { return dirent_; }
	void operator ++ ()
	{
		dirent_ = readdir(dir_);
	}
};



// ----------------------------------------------------------------------

class DosLookUp : public LookUp {
public:
	DosLookUp(char *p, int l) : LookUp(p, l) { }
	virtual char *lookup(char const *) const;
};

char *
DosLookUp::lookup(char const *entry) const
{
	int len, partlen;
	char *path, *dst;
	char const *src;
	char const *srcnext;

	len = strlen(entry);
	path = new char [len_prefix() + 1 + len + 1];
	memcpy(path, prefix(), len_prefix()+1);
	dst = path+len_prefix();
	for (src = entry; src; src=srcnext) {
		srcnext = strpbrk(src, "/\\");
		partlen = srcnext ? srcnext++-src:len;
		len -= partlen+1;

		{
			Dir dir;
			for (dir.open(path); dir; ++dir) {
				if (!strncasecmp(dir->d_name, src, partlen) &&
				    dir->d_name[partlen]=='\0') {
					dst[0] = '/';
					memcpy(&dst[1], dir->d_name,
					       partlen+1);
					break;
				}
			}
			if (!dir) {
				delete path;
				return NULL;
			}
		}
		dst += partlen+1;
	}
	return path;
}


// ----------------------------------------------------------------------

LookUp *LookUpFactory::create_dos(char const *prefix)
{
	char *d = strdup(prefix);
	int len = strlen(prefix);

	return new DosLookUp(d, len);
}
