/*	$AoiMoe: rectset.cc,v 1.4 2001/08/01 20:03:24 tshiozak Exp $	*/

/*
 * Copyright (c) 2001 Takuya SHIOZAKI <AoiMoe@imou.to>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


#include "types.h"
#include "debug.h"
#include "list.h"
#include "rectset.h"


typedef AMList<Rect>::Iterator IterPriv;

void
RectSet::clear()
{
	Rect *p;

	while ((p=rects.remove_head())) {
		delete p;
	}
}

void
RectSet::or_sub(IterPriv elem, XYRect *r)
{
	IterPriv n;
	XYRect i, sub;

	while (elem) {
		i = *elem;

		n = elem;
		++n;
		if (r->x1()>=i.x1() && r->x2()<=i.x2() &&
		    r->y1()>=i.y1() && r->y2()<=i.y2()) {
			/* the rect is inner of current elem */
			/* nothing to do */
			return;
		}
		if ((r->x1()<=i.x1() && r->x2()>=i.x2() &&
		     r->y1()<=i.y1() && r->y2()>=i.y2())) {
			/* current elem is inner of the rect */
			/* merge this elem to the rect */
			delete elem.remove();
			goto next;
		}
		if (r->x2()<i.x1()-1 || r->x1()>i.x2()+1 ||
		    r->y2()<i.y1()-1 || r->y1()>i.y2()+1) {
			/* individual region */
			goto next;
		}
		if (r->x1()==i.x1() && r->x2()==i.x2()) {
			delete elem.remove();
			if (r->y1()<i.y1()) {
				/*
				 *   +------+     +------+    
				 *   | rect |	  | rect |
				 *   +- - - +  -> |      |
				 *   +------+	  |      |
				 *   | elem |	  |      |
				 *   +------+	  +------+
				 */
				r->y2(i.y2());
			} else {
				/*
				 *   +------+     +------+    
				 *   | elem |	  |      |
				 *   +------+	  |      |
				 *   +- - - +  -> |      |
				 *   | rect |	  | rect |
				 *   +------+	  +------+
				 */
				r->y1(i.y1());
			}
			goto next;
		}
		if (r->y1()==i.y1() && r->y2()==i.y2()) {
			delete elem.remove();
			if (r->x1()<i.x1()) {
				r->x2(i.x2());
			} else {
				r->x1(i.x1());
			}
			goto next;
		}
		if (r->x2()<i.x1() || r->x1()>i.x2() ||
		    r->y2()<i.y1() || r->y1()>i.y2()) {
			/* individual region */
			goto next;
		}
		if (r->x1()>=i.x1() && r->x2()<=i.x2()) {
			if (r->y1()<i.y1() && r->y2()<=i.y2()) {
				/*
				 *       +------+             +------+    
				 *       | rect |	      | rect |
				 *   +---|      |---+  -> +---+------+---+
				 *   |   +------+   |	  |              |
				 *   |     elem     |	  |     elem     |
				 *   +--------------+	  +--------------+
				 */
				r->y2(i.y1()-1);
				goto next;
			} else if (r->y1()>=i.y1() && r->y2()>i.y2()) {
				/*
				 *   +--------------+	  +--------------+
				 *   |     elem     |	  |     elem     |
				 *   |   +------+   |	  |              |
				 *   +---|      |---+  -> +---+------+---+
				 *       | rect |	      | rect |
				 *       +------+             +------+    
				 */
				r->y1(i.y2()+1);
				goto next;
			} else {
				/*
				 *       +------+             +------+    
				 *       | rect |	      | rect |
				 *   +--------------+	  +---+------+---+
				 *   |   : elem :   |	  |     elem     |
				 *   +--------------+  -> +---+------+---+
				 *       | rect |	      | rect |
				 *       +------+             +------+    
				 */
				sub.x1(r->x1()); sub.x2(r->x2());
				sub.y1(r->y1()); sub.y2(i.y1()-1);
				or_sub(n, &sub);
				n = elem;
				++n;
				r->y1(i.y2()+1);
				goto next;
			}
		}
		if (r->y1()>=i.y1() && r->y2()<=i.y2()) {
			if (r->x1()<i.x1() && r->x2()<=i.x2()) {
				r->x2(i.x1()-1);
				goto next;
			} else if (r->x1()>=i.x1() && r->x2()>i.x2()) {
				r->x1(i.x2()+1);
				goto next;
			} else {
				sub.y1(r->y1()); sub.y2(r->y2());
				sub.x1(r->x1()); sub.x2(i.x1()-1);
				or_sub(n, &sub);
				n = elem;
				++n;
				r->x1(i.x2()+1);
				goto next;
			}
		}

		if (r->x1()>i.x1()) {
			if (r->y1()>i.y1()) {
				/*
				 * +------+         +------+
				 * | elem |         | elem |
				 * |   +------+  -> |      +---+
				 * +---| rect |     +---+--+---+
				 *     |      |         |      |
				 *     +------+         +------+
				 */
				sub.x1(i.x2()+1); sub.x2(r->x2());
				sub.y1(r->y1()); sub.y2(i.y2());
				or_sub(n, &sub);
				n = elem;
				++n;
				r->y1(i.y2()+1);
				goto next;
			} else {
				/*
				 *     +------+         +--+---+
				 *     |      |         |  |   |
				 * +---| rect |     +---+--+   |
				 * |   +------+  -> |      +---+
				 * | elem |         | elem |
				 * +------+         +------+
				 */
				sub.x1(r->x1()); sub.x2(i.x2());
				sub.y1(r->y1()); sub.y2(i.y1()-1);
				or_sub(n, &sub);
				n = elem;
				++n;
				r->x1(i.x2()+1);
				goto next;
			}
			 
		}
		if (r->y1()>i.y1()) {
			/*
			 *     +------+         +------+
			 *     | elem |         | elem |
			 * +------+   |  -> +---+      |
			 * | rect |+--+     +---+--+---+
			 * |      |         |      |
			 * +------+         +------+
			 */
			sub.x1(r->x1()); sub.x2(i.x1()-1);
			sub.y1(r->y1()); sub.y2(i.y2());
			or_sub(n, &sub);
			n = elem;
			++n;
			r->y1(i.y2()+1);
			goto next;
		} else {
			/*
			 * +------+         +------+
			 * |      |         |      |
			 * | rect |+--+     +---+--+---+
			 * +------+   |  -> +---+      |
			 *     | elem |         | elem |
			 *     +------+         +------+
			 */
			sub.x1(r->x1()); sub.x2(r->x2());
			sub.y1(r->y1()); sub.y2(i.y1()-1);
			or_sub(n, &sub);
			n = elem;
			++n;
			r->y1(i.y1()); r->x2(i.x1()-1);
			goto next;
		}
next:
		elem = n;
	}
	/* add rect to the head of list */
	rects.insert_head(new Rect(*r));
}

RectSet const &
RectSet::operator |= (Rect const &rect)
{
	IterPriv elem;
	XYRect r;

	r = rect;

	elem = rects.first();
	or_sub(elem, &r);

	return *this;
}

RectSet const &
RectSet::operator |= (RectSet const &rs)
{
	for (Iterator i(rs); !i; ++i)
		*this |= i;

	return *this;
}
