/*	$AoiMoe: types.h,v 1.8 2001/08/01 20:03:24 tshiozak Exp $	*/

/*
 * Copyright (c) 2001 Takuya SHIOZAKI <AoiMoe@imou.to>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


#ifndef AMUSE_TYPES_H_
#define AMUSE_TYPES_H_

#include <stdint.h>

typedef uint32_t Tick;

template <class T_>
struct RGBA {
	T_ red_, green_, blue_, alpha_;
	T_ const & r() const { return red_; }
	T_ const & g() const { return green_; }
	T_ const & b() const { return blue_; }
	T_ const & a() const { return alpha_; }
	T_ & r() { return red_; }
	T_ & g() { return green_; }
	T_ & b() { return blue_; }
	T_ & a() { return alpha_; }
};

struct PixelShiftsRec {
	int lshift, rshift;
};
typedef RGBA<PixelShiftsRec> PixelShifts;

class MIPixel {
public:
	typedef uint32_t T_;
private:
	T_ value;
public:
	static T_ const maskR = 0x000000FF;
	static T_ const maskG = 0x0000FF00;
	static T_ const maskB = 0x00FF0000;
	static T_ const maskA = 0xFF000000;
	static T_ const transparentKey = 0;
	static int const shiftR = 0;
	static int const shiftG = 8;
	static int const shiftB = 16;
	static int const shiftA = 24;
	static int const intensityMinR = 0;
	static int const intensityMaxR = 255;
	static int const intensityMinG = 0;
	static int const intensityMaxG = 255;
	static int const intensityMinB = 0;
	static int const intensityMaxB = 255;
	static int const intensityMinA = 0;
	static int const intensityMaxA = 255;
	static int const numColorsR = 256;
	static int const numColorsG = 256;
	static int const numColorsB = 256;
	static int const numColorsA = 256;
	static int const bitsR = 8;
	static int const bitsG = 8;
	static int const bitsB = 8;
	static int const bitsA = 8;
	MIPixel() { }
	explicit MIPixel(T_ v) : value(v) { }
	MIPixel(uint32_t rv, uint32_t gv, uint32_t bv, uint32_t av) :
		value(((rv<<shiftR)&maskR)|((gv<<shiftG)&maskG)|
		      ((bv<<shiftB)&maskB)|((av<<shiftA)&maskA))
	{
	}
	T_ r() const { return (value&maskR)>>shiftR; }
	T_ g() const { return (value&maskG)>>shiftG; }
	T_ b() const { return (value&maskB)>>shiftB; }
	T_ a() const { return (value&maskA)>>shiftA; }
	bool iskey() const { return value==transparentKey; }
};


typedef int Geom;
namespace GeomTraits {
	static Geom const inf = 100000;
}

class Pos {
	Geom posX, posY;
public:
	Pos(Geom ax, Geom ay) : posX(ax), posY(ay) { }
	Pos() { }
	Geom x() const { return posX; }
	Geom y() const { return posY; }
	Geom &x() { return posX; }
	Geom &y() { return posY; }
	void x(Geom v) { posX=v; }
	void y(Geom v) { posY=v; }
	Pos const & operator += (Pos const &pos)
	{
		posX += pos.x();
		posY += pos.y();
		return *this;
	}
	Pos const & operator -= (Pos const &pos)
	{
		posX -= pos.x();
		posY -= pos.y();
		return *this;
	}
};

class WH {
	Geom w_, h_;
public:
	WH(Geom aw, Geom ah) : w_(aw), h_(ah) { }
	WH() { }
	Geom w() const { return w_; }
	Geom h() const { return h_; }
	Geom &w() { return w_; }
	Geom &h() { return h_; }
	void w(Geom v) { w_ = v; }
	void h(Geom v) { h_ = v; }
	bool operator ! () const { return w_<=0 || h_<=0; }
};

class Rect {
	Pos rectPos;
	WH rectWH;
public:
	Rect(Pos const &apos, WH const &awh) :
		rectPos(apos), rectWH(awh)
	{
	}
	Rect(Geom x, Geom y, Geom w, Geom h) :
		rectPos(x, y), rectWH(w, h)
	{
	}
	Rect(Pos const &apos1, Pos const &apos2):
		rectPos(apos1),
		rectWH(apos2.x()-apos1.x()+1, apos2.y()-apos1.y()+1)
	{
	}
	explicit Rect(WH const &wh) :
		rectPos(0, 0), rectWH(wh)
	{
	}

	// note: not a copy constructor
	// default copy constructor will be used.
	template <typename T_>
	Rect(T_ const &r) :
		rectPos(r.x(), r.y()), rectWH(r.w(), r.h())
	{
	}
	Rect() { }

	// note: not a copy assignment op
	// default copy assignment op will be used.
	template <typename T_> Rect const & operator = (T_ const &r)
	{
		rectPos.x(r.x());
		rectPos.y(r.y());
		rectWH.w(r.w());
		rectWH.h(r.h());
		return *this;
	}
	Pos const &to_pos() const { return rectPos; }
	WH const &to_wh() const { return rectWH; }
	Pos &to_pos() { return rectPos; }
	WH &to_wh() { return rectWH; }
	void pos(Pos const &p) { rectPos = p; }
	void wh(WH const &wh) { rectWH = wh; }
	Geom x() const { return rectPos.x(); }
	Geom y() const { return rectPos.y(); }
	Geom w() const { return rectWH.w(); }
	Geom h() const { return rectWH.h(); }
	Geom x1() const { return rectPos.x(); }
	Geom y1() const { return rectPos.y(); }
	Geom x2() const { return rectPos.x()+rectWH.w()-1; }
	Geom y2() const { return rectPos.y()+rectWH.h()-1; }
	Geom &x() { return rectPos.x(); }
	Geom &y() { return rectPos.y(); }
	Geom &w() { return rectWH.w(); }
	Geom &h() { return rectWH.h(); }
	void x(Geom ax) { rectPos.x(ax); }
	void y(Geom ay) { rectPos.y(ay); }
	void w(Geom aw) { rectWH.w(aw); }
	void h(Geom ah) { rectWH.h(ah); }
	void x1(Geom ax) { rectPos.x(ax); }
	void y1(Geom ay) { rectPos.y(ay); }
	void x2(Geom ax) { rectWH.w(ax-rectPos.x()+1); }
	void y2(Geom ay) { rectWH.h(ay-rectPos.y()+1); }
	Rect const & operator |= (Rect const &src);
	Rect const & operator &= (Rect const &dst);
	bool operator ! () const { return !rectWH; }
	Rect const & operator += (Pos const &pos)
	{
		rectPos += pos;
		return *this;
	}
	Rect const & operator -= (Pos const &pos)
	{
		rectPos -= pos;
		return *this;
	}
};

/* operators */
inline bool
hit(Rect const &r, Pos const &p)
{
	return
	    p.x()>=r.x() && p.x()<r.x()+r.w() &&
	    p.y()>=r.y() && p.y()<r.y()+r.h();
}

inline bool
hit(Rect const &r1, Rect const &r2)
{
	return
	    ((r1.x()>=r2.x() && r1.x()<r2.x()+r2.w()) ||
	     (r2.x()>=r1.x() && r2.x()<r1.x()+r1.w())) &&
	    ((r1.y()>=r2.y() && r1.y()<r2.y()+r2.h()) ||
	     (r2.y()>=r1.y() && r2.y()<r1.y()+r1.h()));
}


class XYRect {
	Pos rectPos1, rectPos2;
public:
	XYRect(Pos const &apos1, Pos const &apos2) :
		rectPos1(apos1), rectPos2(apos2)
	{
	}
	XYRect(Pos const &apos, WH const &wh) :
		rectPos1(apos),
		rectPos2(apos.x()+wh.w()-1, apos.y()+wh.h()-1)
	{
	}
	// note: not a copy constructor
	// default copy constructor will be used.
	template <typename T_>
	XYRect(T_ const &r) :
		rectPos1(r.x1(), r.y1()), rectPos2(r.w1(), r.h1())
	{
	}
	XYRect() { }
	// note: not a copy assignment op
	// default copy assignment op will be used.
	template <typename T_> XYRect const & operator = (T_ const &r)
	{
		rectPos1.x(r.x1());
		rectPos1.y(r.y1());
		rectPos2.x(r.x2());
		rectPos2.y(r.y2());
		return *this;
	}
	Geom x1() const { return rectPos1.x(); }
	Geom y1() const { return rectPos1.y(); }
	Geom x2() const { return rectPos2.x(); }
	Geom y2() const { return rectPos2.y(); }
	Geom x() const { return rectPos1.x(); }
	Geom y() const { return rectPos1.y(); }
	Geom w() const { return rectPos2.x()-rectPos1.x()+1; }
	Geom h() const { return rectPos2.y()-rectPos1.y()+1; }
	Geom &x1() { return rectPos1.x(); }
	Geom &y1() { return rectPos1.y(); }
	Geom &x2() { return rectPos2.x(); }
	Geom &y2() { return rectPos2.y(); }
	void x1(Geom ax) { rectPos1.x(ax); }
	void y1(Geom ay) { rectPos1.y(ay); }
	void x2(Geom ax) { rectPos2.x(ax); }
	void y2(Geom ay) { rectPos2.y(ay); }
	void x(Geom ax) { rectPos1.x(ax); }
	void y(Geom ay) { rectPos1.y(ay); }
	void w(Geom aw) { rectPos2.x(rectPos1.x()+aw-1); }
	void h(Geom ah) { rectPos2.y(rectPos1.y()+ah-1); }
};


#endif
