/*	$AoiMoe: vinfo.h,v 1.5 2001/08/01 20:03:24 tshiozak Exp $	*/

/*
 * Copyright (c) 2001 Takuya SHIOZAKI <AoiMoe@imou.to>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


#ifndef AMUSE_VINFO_H_
#define AMUSE_VINFO_H_

class VisualInfo {
	static int get_lsb_addr(uint32_t);
	static int get_norm(uint32_t);
public:
	enum Endian { Big, Little };
protected:
	int bits_per_pixel_;
	int depth_;
	int scanline_pad_;
	bool hasAlpha_;
	Endian endian_;
	PixelShifts shifts_;
	VisualInfo() : bits_per_pixel_(0) { }
	static void calc_shifts(PixelShiftsRec *, uint32_t mask);
public:
	virtual ~VisualInfo() { }
	operator bool() const { return !!bits_per_pixel_; }
	int bits_per_pixel() const { return bits_per_pixel_; }
	int depth() const { return depth_; }
	int scanline_pad() const { return scanline_pad_; }
	bool has_alpha() { return hasAlpha_; }
	Endian endian() const { return endian_; }
	PixelShifts const &shifts() const { return shifts_; }
	bool operator == (VisualInfo const &vi) const
	{
		return this == &vi;
	}
};

class CMap {
	typedef uint32_t Cell;
protected:
	int bitsPerPixel_;
	RGBA<Cell *> element_;
	explicit CMap(RGBA<Cell *> const &elem) :
		element_(elem)
	{
	}
public:
	static int const elementMax = 256;
	virtual ~CMap() { }
	Cell const *r() const { return element_.r(); }
	Cell const *g() const { return element_.g(); }
	Cell const *b() const { return element_.b(); }
	int get_bits_per_pixel() const { return bitsPerPixel_; }
};

class CMapNull : public CMap {
public:
	CMapNull() : CMap(RGBA<Cell *>())
	{
		element_.r() = element_.g() = element_.b() = 0;
	}
};


#endif
