/*	$AoiMoe: sss_ctr.cc,v 1.2 2001/08/08 19:40:30 tshiozak Exp $	*/

/*
 * Copyright (c) 2001 Takuya SHIOZAKI <AoiMoe@imou.to>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "types.h"
#include "debug.h"
#include "vinfo.h"
#include "list.h"
#include "rectset.h"
#include "canvas.h"
#include "event.h"
#include "timeout.h"
#include "servant.h"
#include "lookup.h"
#include "bmp.h"

#pragma implementation "sss_ctr.h"
#include "sss_ini.h"
#include "sss_ctr.h"

class SSSControllerImpl : virtual public SSSController,
			  public ServantStartReceiver,
			  public ServantIdleReceiver,
			  public TimeOutReceiver,
			  public AMButtonReceiver {
	struct Command {
		char const *cmd;
		bool (SSSControllerImpl::*handler)(int) const;
	};
protected:
	static char const * const sectBitmapLabel = "bitmap";
	static char const * const sectWaveLabel = "wave";
	static char const * const sectScenarioLabel = "scnario";
	static Command commands[];
	Servant *servant_;
	ViewPortCanvas dst_;
	SSSIni *ini_;
	SSSIniSection const *sectBitmap_;
	SSSIniSection const *sectScenario_;
	SSSIniSection const *sectWave_;
	int maxScenario_;
	int curNumber_;
	char *prefix_;
	void count_scenario()
	{
		int val;
		maxScenario_ = 0;
		for (AMList<SSSIniElement>::Iterator
			 i(sectScenario_->elements());
		     i; ++i) {
			// XXX: should handle error
			val = atoi((*i).name());
			if (val>maxScenario_) {
				maxScenario_ = val;
			}
		}
		AMUSE_DPR(("maxScenario = %d\n", maxScenario_));
	}
	void post_done()
	{
		servant_->idle_notifier().connect(this);
	}
	bool cmd_bmp(int val)
	{
		char buf[256];
		Canvas *bmp;
		AMList<SSSIniElement>::Iterator i;
		for (i=sectBitmap_->elements(); i; ++i) {
			if (atoi((*i).name())==val)
				break;
		}
		if (i) {
			snprintf(buf, sizeof(buf), "%s/%s",
				 prefix_, (*i).val());
			bmp = BMPCanvasFactory::create(buf);
			if (bmp) {
				bitblt(dst_, Pos(0, 0), *bmp, Pos(0, 0),
				       dst_.wh(), false);
				dst_.sync(true);
				delete bmp;
			} else {
				fill(dst_, Rect(Pos(0, 0), dst_.wh()),
				     MIPixel(0, 0, 0, 255));
				dst_.sync(true);
			}
		} else {
			AMUSE_DPR(("cannot found BMP no. %d\n", val));
		}
		return false;
	}
	bool cmd_wave(int val)
	{
		// bogus
		pid_t pid1, pid2;
		char buf[256];
		AMList<SSSIniElement>::Iterator i;
		for (i=sectWave_->elements(); i; ++i) {
			if (atoi((*i).name())==val)
				break;
		}
		if (i) {
			snprintf(buf, sizeof(buf), "audioplay %s/%s",
				 prefix_, (*i).val());
			if ((pid1=fork())==0) {
				if ((pid2=fork())==0) {
					int i;
					for (i=3; i<20; i++) { // XXX
						close(i);
					}
					for (i=1; i<=10; i++) { // XXX
						if (!system(buf))
							break;
						usleep(10000*i);
					}
					_exit(0);
				}
				_exit(0);
			} else if (pid1>0) {
				int stat;
				waitpid(pid1, &stat, 0);
			}
		}
		return false;
	}
	bool cmd_wait(int val)
	{
		servant_->timeout_notifier().connect(
			this, servant_->current_tick()+val);
		return true;
	}
	void next()
	{
		char const *sio;
		char *p;
		char buf[256];
		int val;
		Command *cmd;
		bool wait = false;

		curNumber_++;
		if (curNumber_>maxScenario_) 
			curNumber_ = 1;

		snprintf(buf, sizeof(buf), "%d", curNumber_);
		sio = sectScenario_->lookup(buf);
		if (sio) {
			snprintf(buf, sizeof(buf), "%s", sio);
			if ((p=strchr(buf, ','))) {
				*p = 0; p++;
				val = atoi(p);
				p = buf;
				while (*p && !isspace(*p))
					p++;
				*p = 0;
				for (cmd=commands; cmd->cmd; cmd++) {
					if (!strcasecmp(cmd->cmd, buf)) {
						AMUSE_DPR((
							"cmd: %s, %d\n",
							buf, val));
						wait =
						    (this->*(cmd->handler))(
							    val);
						break;
					}
				}
				if (!cmd->cmd) {
					AMUSE_DPR(("cannot found command %s\n",
						   buf));
				}
			} else {
				AMUSE_DPR(("cannot find comma"));
			}
		} else {
			AMUSE_DPR(("cannot get scenario %d\n",
				   curNumber_));
		}
		if (!wait)
			post_done();
	}
public:
	SSSControllerImpl(Servant *s) : servant_(s)
	{
		s->start_notifier().connect(this);
		prefix_ = "ehon";
	}
	virtual int mainloop() const
	{
		return servant_->mainloop();
	}
	virtual void callback(ServantStartNotifier const &)
	{
		AMUSE_DPR(("start\n"));
		dst_.bind_canvas(servant_->canvas(),
				 Rect(Pos(0, 0), servant_->canvas().wh()));
		ini_ = SSSIniFactory::create("ehon/ehon_s.scn");
		if (!ini_) {
			AMUSE_DPR(("cannot get ini file\n"));
			return;
		}
		sectScenario_ = ini_->lookup(sectScenarioLabel);
		if (!sectScenario_) {
			AMUSE_DPR(("cannot get scenario section\n"));
			return;
		}
		count_scenario();
		sectBitmap_ = ini_->lookup(sectBitmapLabel);
		if (!sectBitmap_) {
			AMUSE_DPR(("cannot get bitmap section\n"));
		}
		sectWave_ = ini_->lookup(sectWaveLabel);
		if (!sectWave_) {
			AMUSE_DPR(("cannot get wave section\n"));
		}
		curNumber_ = 0;
		servant_->button_notifier().connect(this);
		next();
	}
	virtual void callback(TimeOutNotifier const &)
	{
		next();
	}
	virtual void callback(ServantIdleNotifier const &)
	{
		next();
	}
	virtual void callback(AMButtonNotifier const &, Pos const &)
	{
		exit(0); // XXX
	}
};

SSSControllerImpl::Command SSSControllerImpl::commands[] = {
	{ "bmp", &cmd_bmp },
	{ "wave", &cmd_wave },
	{ "wait", &cmd_wait },
	{ NULL, NULL }
};

SSSController *SSSControllerFactory::create(Servant *s)
{
	return new SSSControllerImpl(s);
}
