/*	$AoiMoe: x11.h,v 1.9 2001/08/08 19:42:13 tshiozak Exp $	*/

/*
 * Copyright (c) 2001 Takuya SHIOZAKI <AoiMoe@imou.to>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


#ifndef AMUSE_X11_P_H_
#define AMUSE_X11_P_H_

typedef uint32_t X11Pixel;

// ----------------------------------------------------------------------

class X11Screen {
protected:
	Display *display_;
	int number_;
public:
	explicit X11Screen(Display *dpy =0, int scr =0) :
		display_(dpy), number_(scr)
	{
	}
	Display *to_display() const { return display_; }
	int to_number() const { return number_; }
	virtual ~X11Screen() { }
	operator bool () const { return !!display_; }
	X11Pixel get_black_pixel() const {
		return BlackPixel(to_display(), to_number());
	}
	X11Pixel get_white_pixel() const {
		return WhitePixel(to_display(), to_number());
	}
};

class X11ScreenReferer {
	X11Screen const &screen_;
protected:
	X11ScreenReferer(X11Screen const &s) : screen_(s) { }
	Display *get_display() const { return screen_.to_display(); }
	int get_screen_number() const { return screen_.to_number(); }
public:
	X11Screen const &to_screen() const { return screen_; }
};

namespace X11ScreenFactory {
	X11Screen *create(char const * =0, int =-1);
};


// ----------------------------------------------------------------------

class X11VisualInfo : public VisualInfo, public X11ScreenReferer {
protected:
	XVisualInfo *visualInfo_;
	X11VisualInfo(X11Screen const &scr, XVisualInfo *vi=NULL) :
		X11ScreenReferer(scr)
	{
		if (vi)
			set_visual_info(vi);
	}
	void set_visual_info(XVisualInfo *);
	bool match_depth(XPixmapFormatValues *, int);
public:
	XVisualInfo const &to_info() const { return *visualInfo_; }
	XVisualInfo const *operator -> () const { return visualInfo_; }
};

namespace X11VisualInfoFactory {
	X11VisualInfo *create(X11Screen const &);
};


// ----------------------------------------------------------------------

class X11Drawable : public X11ScreenReferer {
protected:
	Drawable drawable_;
	X11Drawable(X11Screen const &s, Drawable d) :
		X11ScreenReferer(s), drawable_(d)
	{
	}
public:
	Drawable to_drawable() const { return drawable_; }
	virtual ~X11Drawable() { }
	operator bool () const { return !!drawable_; }
	WH get_wh() const;
};


// ----------------------------------------------------------------------

class X11Window : public X11Drawable {
public:
	X11Window(X11Screen const &s, Window w) :
		X11Drawable(s, (Drawable)w)
	{
	}
	Window to_window() const { return static_cast<Window>(to_drawable()); }
	void map() const { XMapWindow(get_display(), to_window()); }
	void unmap() const { XUnmapWindow(get_display(), to_window()); }
};

namespace X11WindowFactory {
	X11Window *create_simple(X11Screen const &,
				 WH const &,
				 unsigned int =0);
}


// ----------------------------------------------------------------------

class X11GCValues {
	unsigned long mask_;
	XGCValues values_;
public:
	X11GCValues() : mask_(0) { }
	XGCValues const &to_values() const { return values_; }
	unsigned long to_mask() const { return mask_; }
#define X11GCV_ACCESSOR_(n_, m_, t_) \
	void n_(t_ v) { mask_ |= GC##m_; values_.n_ = v; } \
	t_ has_##n_() const { return !!(mask_&GC##m_); } \
	t_ n_() const { return values_.n_; }
	X11GCV_ACCESSOR_(function, Function, int);
	X11GCV_ACCESSOR_(plane_mask, PlaneMask, unsigned long);
	X11GCV_ACCESSOR_(foreground, Foreground, unsigned long);
	X11GCV_ACCESSOR_(background, Background, unsigned long);
	X11GCV_ACCESSOR_(line_width, LineWidth, int);
	X11GCV_ACCESSOR_(line_style, LineStyle, int);
	X11GCV_ACCESSOR_(cap_style, CapStyle, int);
	X11GCV_ACCESSOR_(join_style, JoinStyle, int);
	X11GCV_ACCESSOR_(fill_style, FillStyle, int);
	X11GCV_ACCESSOR_(fill_rule, FillRule, int);
	X11GCV_ACCESSOR_(arc_mode, ArcMode, int);
	X11GCV_ACCESSOR_(tile, Tile, Pixmap);
	X11GCV_ACCESSOR_(stipple, Stipple, Pixmap);
	X11GCV_ACCESSOR_(ts_x_origin, TileStipXOrigin, int);
	X11GCV_ACCESSOR_(ts_y_origin, TileStipYOrigin, int);
	X11GCV_ACCESSOR_(font, Font, Font);
	X11GCV_ACCESSOR_(subwindow_mode, SubwindowMode, int);
	X11GCV_ACCESSOR_(graphics_exposures, GraphicsExposures, Bool);
	X11GCV_ACCESSOR_(clip_x_origin, ClipXOrigin, int);
	X11GCV_ACCESSOR_(clip_y_origin, ClipYOrigin, int);
	X11GCV_ACCESSOR_(clip_mask, ClipMask, Pixmap);
	X11GCV_ACCESSOR_(dash_offset, DashOffset, int);
	// dash list
	static X11GCValues const none;
};

class X11GC : public X11ScreenReferer {
protected:
	GC gc_;
	X11GC(X11Drawable const &d, GC gc =0) :
		X11ScreenReferer(d.to_screen()), gc_(gc)
	{
	}
public:
	GC to_gc() const { return gc_; }
	virtual ~X11GC() { }
	operator bool () const { return !!gc_; }
};

namespace X11GCFactory {
	X11GC *create(X11Drawable const &,
		      X11GCValues const & =X11GCValues::none);
};


// ----------------------------------------------------------------------

namespace CMapFactory {
	CMap *create(X11VisualInfo const &);
}


// ----------------------------------------------------------------------

class X11Canvas;

class X11CanvasImpl : X11ScreenReferer {
	friend class X11Canvas;
protected:
	typedef bool (*MatchFunc)(X11Screen const &);
	typedef X11CanvasImpl *(*Factory)(X11Canvas const &,
					  X11Screen const &,
					  X11Drawable const &);
	struct MatchContext {
		MatchFunc match;
		Factory factory;
	};
	static MatchContext const *matchContextList_[];
	X11Canvas const &canvas_;
	X11Drawable const &drawable_;
	X11GC *gc_;
	XImage *image_;
	X11CanvasImpl(X11Canvas const &c,
		      X11Screen const &s,
		      X11Drawable const &d,
		      X11GC *gc=0, XImage *img=0) :
		X11ScreenReferer(s),
		canvas_(c), drawable_(d), gc_(gc), image_(img)
	{
	}
	static X11CanvasImpl *create(X11Canvas const &,
				     X11Screen const &,
				     X11Drawable const &);
public:
	virtual ~X11CanvasImpl() { }
	virtual void sync(bool) const = 0;
	int bytes_per_line() const { return image_?image_->bytes_per_line:0; }
};

class X11Canvas : public MDCanvas, public SimpleCanvasRegionImpl {
protected:
	X11CanvasImpl *impl_;
	X11Canvas(X11VisualInfo const &vinfo, CMap const &cmap, WH const &wh) :
		MDCanvas(vinfo, cmap, wh), impl_(0)
	{
	}
	static X11CanvasImpl *create(X11Canvas const &c,
				     X11Screen const &s,
				     X11Drawable const &d)
	{
		return X11CanvasImpl::create(c, s, d);
	}
public:
	X11VisualInfo const &visual_info() const
	{
		return
		    static_cast<X11VisualInfo const &>(
			    this->MDCanvas::visual_info());
	}
	operator bool () const { return !!impl_; }
	virtual void *get_frame_raw() const { return impl_->image_->data; }
	virtual void sync(bool isclear) const { return impl_->sync(isclear); }
};

namespace MDCanvasFactory {
	MDCanvas *create(X11Drawable const &,
			 X11VisualInfo const &, CMap const &);
}


#endif
