/*	$AoiMoe: xaw_p.h,v 1.10 2001/08/01 20:03:34 tshiozak Exp $	*/

/*
 * Copyright (c) 2001 Takuya SHIOZAKI <AoiMoe@imou.to>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */


#ifndef AMUSE_XAW_P_H_
#define AMUSE_XAW_P_H_

class XawWidget {
	Widget widget_;
public:
	explicit XawWidget(Widget w) : widget_(w) { }
	virtual ~XawWidget() { }
	Widget to_widget() const { return widget_; }
	operator bool () const { return !!widget_; }
	XtAppContext to_appcon() const
	{
		return XtWidgetToApplicationContext(widget_);
	}
	Display *to_display() const
	{
		return XtDisplay(widget_);
	}
	Screen *to_screen() const
	{
		return XtScreen(widget_);
	}
	Window to_window() const
	{
		return XtWindow(widget_);
	}
};

class XawServant;
class XawIdleNotifier : public ServantIdleNotifier {
protected:
	XawServant const &servant_;
	static Boolean work_proc(XtPointer);
public:
	XawIdleNotifier(XawServant const &sv) : servant_(sv) { }
	virtual void notify(ServantIdleNotifier const &)
	{
		notify_base(*this);
	}
	virtual void connect(R_ *rv);
	virtual void disconnect(R_ *rv) { disconnect_base(rv); }
};

class XawTimeOutNotifier : public TimeOutNotifier {
	struct ClientData {
		XawTimeOutNotifier *notifier;
		XtIntervalId xtid;
	};
	XtAppContext appcon_;
	mutable int serial_;
	static void timeout_callback(XtPointer, XtIntervalId *);
public:
	XawTimeOutNotifier(XtAppContext ac) : appcon_(ac), serial_(1) { }
	virtual void connect(TimeOutReceiver *, Tick);
	virtual void disconnect(TimeOutReceiver *);
};

class XawServant : public Servant {
	mutable X11Screen *screen_;
	mutable X11VisualInfo *vinfo_;
	mutable CMap *cmap_;
	mutable X11Window *window_;
	mutable bool opened_;
	mutable XawIdleNotifier idleNotifier_;
	mutable SimpleAMButtonNotifier buttonNotifier_;
	mutable SimpleAMMotionNotifier motionNotifier_;
	XawWidget *shell_, *mainWindow_;
	static struct timeval epoch;
	static void get_epoch();
public:
	XawServant(int *, char **, WH const &);
	XtAppContext to_appcon() const { return shell_->to_appcon(); }
	XawWidget const &shell() const { return *shell_; }
	operator bool () const { return !!shell_; }
	//
	virtual int mainloop() const;
	virtual ServantIdleNotifier &idle_notifier() const;
	virtual AMButtonNotifier &button_notifier() const;
	virtual AMMotionNotifier &motion_notifier() const;
	virtual Pos get_cursor_pos() const;
	virtual Tick current_tick() const;
	static Tick xaw_current_tick();
	//
	static void expose_proc(Widget, XEvent *);
	void expose_cb(Rect const &);
	static void button_proc(Widget, XEvent *);
	void button_cb(Pos const &);
	static void motion_proc(Widget, XEvent *);
	void motion_cb(Pos const &);
};


#endif
